---
title: "gRPC and AI: A Powerful Partnership"
date: 2025-05-20
---

We recently showcased the exciting potential of Large Language Models (LLMs) to transform gRPC development at gRPConf 2024\. If you're curious to see how AI can streamline your gRPC workflows, read on\! We would love to emphasize that these capabilities are accessible to you *today* using [Gemini Code Assist](https://codeassist.google/).

In our gRPConf demonstration, we explored how LLMs, particularly Google's Gemini, can streamline your gRPC workflows in three key ways. Now let's see how can you utilize LLM's gRPC knowledge using Gemini Code Assist as an example:

1. **Swagger to Protobuf: Explore Your API as a gRPC Service:** Curious to see what your existing REST APIs would look like as high-performance gRPC services? Open your OpenAPI YAML file, then write prompts like 

   *"Convert OpenAPI schema to Protobuf"* 

   to explore the potential of gRPC for your current services, accelerating adoption, and discovering new possibilities for your API architecture.

2. **Protobuf to JUnit Test Cases: Supercharge Your Testing Workflow:** Want to ensure your gRPC services are robust and reliable? You can use prompts like 

   *"Create JUnit tests for a service implementing PetStoreService in petstore.proto"* 

   to automatically generate comprehensive JUnit test cases from your Protobuf definitions. Experience how you can quickly create tests in multiple languages (Java, Go, Python, etc.), enabling you to focus on building innovative features with confidence.

3. **Natural Language to Protobuf: Unleash Your Service Ideas:** Have a vision for a new gRPC service? You can directly describe your service concepts in plain English, using prompts like 

   *"Help me create a pet store gRPC service, and it should list all the pets, create a new pet, and get pet information by ID."* 

   to generate Protobuf schemas. Discover how you can quickly prototype new services, foster collaboration between teams, and bring your ideas to life with ease. 

LLMs like Gemini possess a significant understanding of gRPC concepts and code structures, enabling you to leverage them in your development workflow. We're excited to continue exploring the potential of LLMs to further streamline workflows, automate tasks, and empower developers to build and maintain high-quality gRPC services.

At the same time, feel free to start trying Gemini Code Assist with your own prompts today\!

Catch the [recording of our demo](https://www.youtube.com/watch?v=IZsYnCftm7w) in the below YouTube video to see these prompt-based transformations in action. And don't miss the opportunity to connect with the gRPC community and explore more innovations at the upcoming gRPC Conference on August 26th, Sunnyvale, CA. You can

* Sign up for the upcoming gRPC Conference in 2025 at [https://events.linuxfoundation.org/grpconf/](https://events.linuxfoundation.org/grpconf/)   
* Submit a talk proposal before May 28th, 11:59 PM PDT at: [https://events.linuxfoundation.org/grpconf/program/cfp/](https://events.linuxfoundation.org/grpconf/program/cfp/)

Stay tuned for more exciting advancements in the gRPC and AI space at the upcoming gRPC Conference in 2025\! 
